package frc.robot.constants;

import static edu.wpi.first.units.Units.Volts;
import static edu.wpi.first.units.Units.Second;
import static edu.wpi.first.units.Units.Seconds;

import com.ctre.phoenix6.configs.CANcoderConfiguration;
import com.ctre.phoenix6.signals.SensorDirectionValue;
import com.revrobotics.spark.config.SparkMaxConfig;
import com.revrobotics.spark.config.SparkBaseConfig.IdleMode;

import edu.wpi.first.math.util.Units;
import edu.wpi.first.wpilibj2.command.sysid.SysIdRoutine;
import edu.wpi.first.wpilibj2.command.sysid.SysIdRoutine.Config;

public class ManipulatorPivotConstants {
    public static final int kArmMotorID = 0;
    public static final int kCANcoderID = 0;

    public static final int kMotorAmpsMax = 40;

    public static final double kPivotConversion = 12/60 * 20/60 * 28/12;

    public static final double kPivotMaxVelocity = 0;

    public static final double kPositionalP = 0;
    public static final double kPositionalI = 0;
    public static final double kPositionalD = 0;
    public static final double kPositionalTolerance = Units.degreesToRadians(1);

    public static final double kVelocityP = 0;
    public static final double kVelocityI = 0;
    public static final double kVelocityD = 0;

    public static final double kFeedForwardS = 0;
    public static final double kFeedForwardG = 0.41; // calculated value
    public static final double kFeedForwardV = 0.68; //calculated value

    // TODO Is this reasonable?
    public static final double kVelocityTolerance = Units.degreesToRadians(3) / 60;

    public static final double kCoralIntakePosition = 0;
    public static final double kL1Position = 0;
    public static final double kL2Position = 0;
    public static final double kL3Position = 0;
    public static final double kL4Position = 0;
    public static final double kL2AlgaePosition = 0;
    public static final double kL3AlgaePosition = 0;
    /**The closest position to the elevator brace without hitting it */
    public static final double kArmSafeStowPosition = 0;
    /**The forward rotation limit of the arm */
    public static final double kRotationLimit = 0;

    public static final double kMagnetOffset = 0.0;
    public static final double kAbsoluteSensorDiscontinuityPoint = 0.0;

    public static final double kSysIDRampRate = 1;
    public static final double kSysIDStepVolts = 7;
    public static final double kSysIDTimeout = 10;

    public static final SensorDirectionValue kSensorDirection = SensorDirectionValue.CounterClockwise_Positive;

    public static final IdleMode kIdleMode = IdleMode.kBrake;

    // YOU SHOULDN'T NEED TO CHANGE ANYTHING BELOW THIS LINE UNLESS YOU'RE ADDING A NEW CONFIG

    public static final SysIdRoutine.Config kSysIDConfig = new Config(
        Volts.of(kSysIDRampRate).per(Second),
        Volts.of(kSysIDStepVolts),
        Seconds.of(kSysIDTimeout)
    );

    public static final CANcoderConfiguration canCoderConfig = new CANcoderConfiguration();

    public static final SparkMaxConfig motorConfig = new SparkMaxConfig();

    static {
        canCoderConfig.MagnetSensor.SensorDirection = SensorDirectionValue.CounterClockwise_Positive;
        canCoderConfig.MagnetSensor.MagnetOffset = kMagnetOffset;
        // TODO Need to do more reading on this setting, and how to properly offset the Arm so that horizontal is 0
        //canCoderConfig.MagnetSensor.AbsoluteSensorDiscontinuityPoint = 0.5;

        motorConfig
            .smartCurrentLimit(kMotorAmpsMax)
            .idleMode(kIdleMode);
        motorConfig.encoder.positionConversionFactor(kPivotConversion);
    }


}
